/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "chunk.h"

chunklist_t * chunksCreate(uint32_t width,uint32_t height, uint32_t depth) {
  chunklist_t *chunks;
  uint32_t i;
  
  chunks = malloc(sizeof(chunklist_t));
  if(!chunks) return NULL;

  chunks->width = width;
  chunks->height = height;
  chunks->depth = depth;
  chunks->count = width * height * depth;

  chunks->indexes = malloc(sizeof(uint32_t) * chunks->count);
  if(!chunks->indexes) {
    free(chunks);
    return NULL;
  }

  for(i = 0; i < chunks->count; i++) chunks->indexes[i] = i;

  return chunks;
}

void chunksDispose(chunklist_t *list) {
  free(list->indexes);
  free(list);
}

void chunksShift(chunklist_t *list, uint32_t direction[3]) {
  uint32_t i, x, y, z, my, mz;

  // Precalculate these values
  my = list->width * list->height;
  mz = my * list->depth;

  // For each chunk
  for(i = 0; i < list->count; i++) {
    // Precalculate, store here for now.
    x = i / list->width;

    // Calculate the new xyz for the current index.
    y = (x + direction[0]) % list->height;
    z = (x / list->height + direction[1]) % list->depth;
    x = (i + direction[2]) % list->width;

    // Set back into the indexes array.
    list->indexes[i] = (x * list->width) + (y * my) + (z * mz);
  }
}