// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <stdint.h>
#include <malloc.h>

typedef struct {
  uint32_t width, height, depth;
  uint32_t count;
  uint32_t *indexes;
} chunklist_t;

/**
 * Create a new chunk manager.
 * 
 * @param width Width of the chunk list.
 * @param height Height of the chunk list.
 * @param depth Depth of the chunk list.
 * @returns The new chunk list manager
 */
chunklist_t * chunksCreate(uint32_t width,uint32_t height, uint32_t depth);

/**
 * Cleans up a previously created chunk list.
 * 
 * @param list List to dispose.
 */
void chunksDispose(chunklist_t *list);

/**
 * Shift the chunk list along a set of axis.
 * 
 * @param list List to shift.
 * @param direction Array of directions to shift along.
 */
void chunksShift(chunklist_t *list, uint32_t direction[3]);